﻿// Copyright 2009 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Scripting.Hosting;
using IronPython.Hosting;
using System.Scripting.Actions;
using System.Linq.Expressions;

namespace Dynamic {
  class Program {
    static void Main(string[] args) {
      dynamic i = 42;
      dynamic s = "Hi there";

      // calling stuff on dynamic instances isn't checked at compile time
      //i.DoSomethingImpossible();

      Console.WriteLine("-------------------------------------");
      PythonScript();

      Console.WriteLine("-------------------------------------");
      DoFall(new Leaf());
      DoFall(new ExchangeRate());

      // This should throw an exception, but for some reason it doesn't always do that -
      // Try the second block below to see what should happen.
      DoFall(42);

      //dynamic fiftytwo = 52;
      //fiftytwo.Fall();

      Console.WriteLine("-------------------------------------");
      foreach (var val in GetStuff())
        Console.WriteLine(val);

      Console.WriteLine("-------------------------------------");
      dynamic myObject = new MyDynamicType();
      myObject.Blurb();
      myObject.Argh();
      myObject.Oops();
      DoFall(myObject);

    }

    private static void PythonScript( ) {
      Console.WriteLine("Loading script.py...");
      ScriptRuntime python = Python.CreateRuntime();
      dynamic script = python.UseFile("script.py");
      Console.WriteLine("script.py loaded!");

      script.doSomething();
    }

    static IEnumerable<dynamic> GetStuff( ) {
      yield return "a string";
      yield return 42;
      yield return true;
    }

    // For some odd reason, the exception isn't thrown in this method
    // if the thing doesn't have a Fall method. Still left it in, because
    // this is how it should be :-)
    static void DoFall(dynamic thing) {
      try {
        thing.Fall();
      }
      catch {
        Console.WriteLine("Turns out {0} can't fall.", thing);
      }
    }
  }

  public class Leaf {
    public void Fall( ) {
      Console.WriteLine("Leaf is falling");
    }
  }

  public class ExchangeRate {
    public Action Fall {
      get {
        return ( ) =>
          Console.WriteLine("Exchange rate is falling");
      }
    }
  }

  public class MyDynamicType : IDynamicObject {

    MetaObject IDynamicObject.GetMetaObject(System.Linq.Expressions.Expression parameter) {
      return new MyDynamicTypeMetaObject(parameter);
    }

    public class MyDynamicTypeMetaObject : MetaObject {
      public MyDynamicTypeMetaObject(Expression expression) : this(expression, Restrictions.Empty) { }
      public MyDynamicTypeMetaObject(Expression expression, Restrictions restrictions) : 
        base(expression, restrictions) { }

      public MyDynamicTypeMetaObject(Expression expression, Restrictions restrictions, object value) :
        base(expression, restrictions, value) { }

      public override MetaObject Call(CallAction action, MetaObject[] args) {
        Console.WriteLine("Action " + action.Name + " invoked");
        return this;
      }
    }
  }
}
