﻿// Copyright 2009 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Office.Interop;
using Excel = Microsoft.Office.Interop.Excel;

namespace Parameters {
  class Program {
    static void Main(string[] args) {
      DoSomething(5);
      DoSomething(x: 42);
      DoSomething(5, 3);
      DoSomething(y: 100, x: 13);
      DoSomething(y: GetY(), x: GetX());

      DoSomethingElse(z: 33);

      var man = new Man();
      man.Walk();
      var human = (Human) man;
      human.Walk();

      human.Calculate();
      man.Calculate();

      Automation();
    }

    static void DoSomething(int x) {
      Console.WriteLine("One param DoSomething: x={0}", x);
    }

    static void DoSomething(int x, int y = 20) {
      Console.WriteLine("Two param DoSomething: x={0}, y={1}", x, y);
    }

    static int GetX( ) {
      Console.WriteLine("Getting X");
      return 10;
    }

    static int GetY( ) {
      Console.WriteLine("Getting Y");
      return 20;
    }

    static void DoSomethingElse(int x = 10, int y = 20, int z = 30) {
      Console.WriteLine("DoSomethingElse: x={0}, y={1}, z={2}", x, y, z);
    }


    static void Automation( ) {
      var excel = new Excel.Application();

      var workbook = excel.Workbooks.Add();
      var sheet = excel.ActiveSheet;

      excel.Visible = true;
      // Cells returns a dynamic
      excel.Cells[1, 1] = "Hi from C#!";
      excel.Columns[1].AutoFit();

      excel.Cells[2, 1] = 10;
      excel.Cells[3, 1] = 15;
      excel.Cells[4, 1] = 20;
      excel.Cells[5, 1] = 30;
      excel.Cells[6, 1] = 8;

      var chart = workbook.Charts.Add(After: sheet);
      // This next line should configure the chart to actually display
      // something, but right now I get the error message 
      // "Dynamically bound calls may not contain named arguments (feature not yet implemented)"
      // Too bad.

      //chart.ChartWizard(Source: sheet.get_Range("A2", "A6"));
    }
  }

  public class Human {
    public virtual void Walk( ) {
      Console.WriteLine("Human walking");
    }

    public void Calculate(int x = 20, int y = 30) {
      Console.WriteLine("Human calculating");
    }
  }

  public class Man : Human {
    public override void Walk( ) {
      Console.WriteLine("Man walking");
    }

    public void Calculate(int y = 20, int x = 30) {
      Console.WriteLine("Man calculating");
    }
  }
}
